/*+
|          Sistemas Integrados TEKNISA       |
|                  Core Vision               |
|                                            |
|       Procedure: ETL_COMERCIALTEK          |
|                                            |  
|    Etl para apurao de contatos do dia 21 |
|   de um ms ao dia 20 do outro.            |
|       Preenche a tabela ETLCOMERCIALDT     |
|                                            |
|     Cadastramento da procedure no AEV     |
|                                            |
|                                            |
|    Cadastramento dos fatos e dimenses    |
|            no Core Vision                  |
|                                            |
|       Criao:                             |
|       30/06/2008 - Douglas Lelis Ribeiro   |
|                                            |            	              
|                 			     |
|                                            |                                                 			|
|                                            |
+*/


create or replace
PROCEDURE ETL_COMERCIALTEK
                     (P_DTINI  IN DATE,
                      P_DTFINA IN DATE)
IS
  V_DTFIM           DATE;
  V_DTFINA          DATE;
  V_DTINI           DATE;  
  V_CDSEQUENCIAL    VARCHAR2(20);
  V_NIVEL           VARCHAR2(20);
  V_DTMES           VARCHAR2(07);
  V_DTANO           VARCHAR2(04);
  V_DSPERIODO       VARCHAR2(15);  
  V_IDCONTAPROV     VARCHAR2(10);
  V_IDSTATUSCONT    VARCHAR2(12);
  V_NMCONTACTADO    VARCHAR2(100);
  V_TIPOCONTATADO   VARCHAR2(10);
  V_CDPAIS          PAIS.CDPAIS%TYPE;
  V_SGESTADO        ESTADO.SGESTADO%TYPE;
  V_CDMUNICIPIO     MUNICIPIO.CDMUNICIPIO%TYPE;
  /*CONSTANTES*/
  CON_SIM           VARCHAR2(3)  := 'Sim';
  CON_NAO           VARCHAR2(3)  := 'No';
  CON_NAO_LIDO      VARCHAR2(10) := 'No Lido';
  CON_CONCLUIDO     VARCHAR2(9)  := 'Concludo';
  CON_ANDAMENTO     VARCHAR2(12) := 'Em Andamento';
  CON_PROSPECT      VARCHAR2(08) := 'Prospect';
  CON_CLIENTE       VARCHAR2(07) := 'Cliente';
  CON_FORNECEDOR    VARCHAR2(10) := 'Fornecedor';
  CON_VENDEDOR      VARCHAR2(08) := 'Vendedor';
  FUNCTION MES_NOME (P_DTMES IN VARCHAR2) RETURN VARCHAR2
  AS
     V_RESULT       VARCHAR2(3);
  BEGIN  
     SELECT DECODE(P_DTMES,'01','Jan','02','Fev','03','Mar','04','Abr','05','Mai',
                           '06','Jun','07','Jul','08','Ago','09','Set','10','Out','11','Nov','Dez')
     INTO V_RESULT 
     FROM DUAL;
     
     RETURN(V_RESULT);  
  END;  
  
BEGIN
    IF TO_CHAR(P_DTINI,'DD') <>'21' THEN       
       IF TO_NUMBER(TO_CHAR(P_DTINI,'DD')) < 21 THEN             
          V_DTINI  := '21/' || TO_CHAR(ADD_MONTHS(P_DTINI,-1),'MM/YYYY');--V_DTINI receber a data do dia 21 anterior mais prximo
       ELSE
          V_DTINI  := '21/' || TO_CHAR(P_DTINI,'MM/YYYY'); 
       END IF;   
    ELSE
       V_DTINI  :=  P_DTINI;
    END IF;   
    
    V_DTMES  := TO_CHAR(V_DTINI,'MM');
    V_DTANO  :=  TO_CHAR(V_DTINI,'YYYY');
    
/*APAGA OS DADOS J EXISTENTES A PARTIR DA DATA INCIO*/
    DELETE FROM ETLCOMERCIALDT
    WHERE  (ANFATETLCOM > V_DTANO) OR --Deletar os dados posteriores a data parmetro
          (ANFATETLCOM = V_DTANO AND 
          --Converte a sigla do ms em numeral
          DECODE(SUBSTR(DSPERIFATETLCOM,4,3),'Jan','01','Fev','02','Mar','03','Abr','04','Mai','05',
                                             'Jun','06','Jul','07','Ago','08','Set','09','Out','10','Nov','11','12')
          >= V_DTMES);
    
    IF TO_CHAR(P_DTFINA,'DD') <>'20' THEN       
       IF TO_NUMBER(TO_CHAR(P_DTFINA,'DD')) > 20 THEN          
          V_DTFINA  := '20/' || TO_CHAR(ADD_MONTHS(P_DTFINA,1),'MM/YYYY');--V_DTFINA receber a data do dia 20 posterior mais prximo
       ELSE
          V_DTFINA  := '20/' || TO_CHAR(P_DTINI,'MM/YYYY');
       END IF;   
    ELSE
       V_DTFINA  :=  P_DTFINA;
    END IF;     
    
    WHILE (V_DTINI  <  V_DTFINA) LOOP       
       V_DTFIM := '20/' || TO_CHAR(ADD_MONTHS(V_DTINI,1),'MM/YYYY') ; 

       FOR C_ETL IN
             (SELECT COUNT(CON.CDFILIAL||CON.NRCONTATO||EVO.DTHREVOLCONT) AS QTCONTATOS, 
                    CON.CDMOTICONT, CON.IDSTATUSCONT, CON.CDCLIENTE,    CON.CDFILIAL,
                    CON.CDFORNECED, CON.CDPROSPECT,   CON.CDVENDEDOR,   CON.IDTPCONTATO,
                    CON.CDSISTEMA,  CON.CDOPCONCCONT, CON.IDCONTAPROV,  CON.CDMTENCECONT, 
                    EVO.CDOPEREVOL, EVO.CDMOTIEVOCON, SIS.CDPROCOMERCI
              FROM
                    CONTATO CON, EVOLCONT EVO, SISTEMA SIS
              WHERE
                    (TRUNC(EVO.DTHREVOLCONT)  BETWEEN V_DTINI AND V_DTFIM)
                AND (EVO.CDFILIAL (+) = CON.CDFILIAL)
                AND (EVO.NRCONTATO(+) = CON.NRCONTATO)
                AND (SIS.CDSISTEMA(+) = CON.CDSISTEMA)
                
              GROUP BY
                    CON.CDFILIAL,    CON.CDMOTICONT,   CON.IDSTATUSCONT,
                    CON.CDCLIENTE,   CON.CDFORNECED,   CON.CDPROSPECT,     CON.CDVENDEDOR,
                    CON.IDTPCONTATO, CON.CDSISTEMA,    CON.CDOPCONCCONT, CON.IDCONTAPROV,
                    EVO.CDOPEREVOL,  EVO.CDMOTIEVOCON, SIS.CDPROCOMERCI, CON.CDMTENCECONT
              )
       LOOP
/* SOMENTE QUALIFICA COMO APROVADO OU NAO APROVADO OS CONTATOS CUJO MOTIVO DA EVOLUO SEJA QUALIFICAO.
CASO CONTRARIO, TODOS OS CONTATOS SO CLASSIFICADOS COMO NO LIDOS */
              IF C_ETL.CDMOTIEVOCON<>'12' THEN
                 V_IDCONTAPROV := CON_NAO_LIDO;
              ELSE
                 IF C_ETL.IDCONTAPROV = 'S' THEN
                    V_IDCONTAPROV := CON_SIM;
                 ELSIF C_ETL.IDCONTAPROV = 'N' THEN
                    V_IDCONTAPROV := CON_NAO;
                 ELSE V_IDCONTAPROV := CON_NAO_LIDO;
                 END IF;
              END IF;

              IF C_ETL.IDSTATUSCONT = 'C' THEN
                 V_IDSTATUSCONT := CON_CONCLUIDO;
              ELSE
                 V_IDSTATUSCONT := CON_ANDAMENTO;
              END IF;

/*NOME, PAS, ESTADO E MUNICPIO DO CONTATADO*/
              IF C_ETL.IDTPCONTATO='C' THEN
                 V_TIPOCONTATADO  := CON_CLIENTE;
                 BEGIN
                    SELECT NMFANTCLIE
                    INTO V_NMCONTACTADO
                    FROM CLIENTE
                    WHERE CDCLIENTE = C_ETL.CDCLIENTE;
                 EXCEPTION
                 WHEN OTHERS THEN
                    V_NMCONTACTADO := NULL;
                 END;
                 
                 BEGIN
                    SELECT CDPAIS,SGESTADO,CDMUNICIPIO
                    INTO V_CDPAIS,V_SGESTADO,V_CDMUNICIPIO
                    FROM ENDECLIE
                    WHERE (CDCLIENTE = C_ETL.CDCLIENTE) AND
                    (IDTPENDECLIE = 'P');
                 EXCEPTION
                 WHEN OTHERS THEN
                    BEGIN
                       V_CDPAIS := NULL;
                       V_SGESTADO := NULL;
                       V_CDMUNICIPIO := NULL;
                    END;
                 END;
              ELSIF C_ETL.IDTPCONTATO='P' THEN
                    V_TIPOCONTATADO  := CON_PROSPECT;
                    BEGIN
                       SELECT NMFANTPROS
                       INTO V_NMCONTACTADO
                       FROM PROSPECT
                       WHERE (CDPROSPECT = C_ETL.CDPROSPECT)  ;
                       EXCEPTION
                       WHEN OTHERS THEN
                          V_NMCONTACTADO := NULL;
                    END;
                    
                    BEGIN
                       SELECT CDPAIS,SGESTADO,CDMUNICIPIO
                       INTO V_CDPAIS,V_SGESTADO,V_CDMUNICIPIO
                       FROM ENDEPROSPECT
                       WHERE CDPROSPECT = C_ETL.CDPROSPECT AND
                       (IDTPENDEPROS = 'P');
                       EXCEPTION WHEN OTHERS THEN BEGIN
                          V_CDPAIS := NULL;
                          V_SGESTADO := NULL;
                          V_CDMUNICIPIO := NULL;
                       END;
                    END;
              ELSIF C_ETL.IDTPCONTATO='F' THEN
                    V_TIPOCONTATADO  := CON_FORNECEDOR;
                    BEGIN
                       SELECT NMFANTFORN
                       INTO V_NMCONTACTADO
                       FROM FORNECEDOR
                       WHERE (CDFORNECED = C_ETL.CDFORNECED) ;
                       EXCEPTION
                       WHEN OTHERS THEN
                          V_NMCONTACTADO := NULL;
                    END;
                    
                    BEGIN
                       SELECT CDPAIS,SGESTADO,CDMUNICIPIO
                       INTO V_CDPAIS,V_SGESTADO,V_CDMUNICIPIO
                       FROM ENDEFORN
                       WHERE CDFORNECED = C_ETL.CDFORNECED AND
                       (IDTPENDEFORN = 'P');
                       EXCEPTION WHEN OTHERS THEN BEGIN
                          V_CDPAIS := NULL;
                          V_SGESTADO := NULL;
                          V_CDMUNICIPIO := NULL;
                       END;
                    END;
              ELSIF C_ETL.IDTPCONTATO='V' THEN
                    V_TIPOCONTATADO  := CON_VENDEDOR;
                    BEGIN
                       SELECT NMFANVEN
                       INTO V_NMCONTACTADO
                       FROM VENDEDOR
                       WHERE CDVENDEDOR = C_ETL.CDVENDEDOR;
                       EXCEPTION
                       WHEN OTHERS THEN
                          V_NMCONTACTADO := NULL;
                    END;
                    
                    BEGIN
                       SELECT CDPAIS,SGESTADO,CDMUNICIPIO
                       INTO   V_CDPAIS,V_SGESTADO,V_CDMUNICIPIO
                       FROM   ENDEVENDE
                       WHERE  CDVENDEDOR = C_ETL.CDVENDEDOR
                          AND IDTPENDEVEN = 'P';
                       EXCEPTION WHEN OTHERS THEN BEGIN
                          V_CDPAIS := NULL;
                          V_SGESTADO := NULL;
                          V_CDMUNICIPIO := NULL;
                       END;
                    END;
              END IF;

              V_DTMES  :=  MES_NOME(TO_CHAR(V_DTINI,'MM'));
              V_DTANO  :=  TO_CHAR(V_DTINI,'YYYY');
              V_DSPERIODO := '21/' || V_DTMES ||' a ';
              V_DTMES  :=  MES_NOME(TO_CHAR(V_DTFIM,'MM'));
              V_DSPERIODO := V_DSPERIODO || '20/' || V_DTMES;              
              
              INSERT INTO ETLCOMERCIALDT
                      (CDFILIAL,        CDMOTICONT,      CDSISTEMA,
                       CDOPEREVOL,      CDMOTIEVOCON, 
                       CDPAIS,          SGESTADO,        CDMUNICIPIO,
                       CDPROCOMERCI,                     CDMTENCECONT,
                       DSCONTAPRETLCOM,                  DSSTATETLCOM,
                       DSTPCONTETLCOM,                   ANFATETLCOM,           
                       NMCONTATOETLCOM, DSPERIFATETLCOM, QTCONTETLCOM 
                      )
              VALUES
                      (C_ETL.CDFILIAL,        C_ETL.CDMOTICONT,      C_ETL.CDSISTEMA,
                       C_ETL.CDOPEREVOL,      C_ETL.CDMOTIEVOCON,
                       V_CDPAIS,              V_SGESTADO||V_CDPAIS,  V_CDMUNICIPIO||V_SGESTADO||V_CDPAIS,
                       C_ETL.CDPROCOMERCI,    C_ETL.CDMTENCECONT,
                       V_IDCONTAPROV,         V_IDSTATUSCONT, 
                       V_TIPOCONTATADO,       V_DTANO,
                       V_NMCONTACTADO,        V_DSPERIODO,           C_ETL.QTCONTATOS                                           
                      );
       END LOOP;

       COMMIT;
       V_DTINI := ADD_MONTHS(V_DTINI,1);
    END LOOP;
END;
/

DECLARE
   V_CDSEQUENCIAL VARCHAR2(20) NULL;
BEGIN
    /*REGISTRANDO A PROCEDURE PARA O AGENDADOR DE EVENTOS*/
    SELECT COUNT(*)
    INTO   V_CDSEQUENCIAL
    FROM   EVENTOAEV 
    WHERE  NMOBJBDEVEN = 'ETL_COMERCIALTEK';
    
    IF V_CDSEQUENCIAL = 0 THEN     --> S ADICIONA A PROCEDURE SE ELA NO EXISTIR AINDA NO CADASTRO
        BEGIN
        
            SELECT LPAD(NVL(MAX(CDEVENTOAEV)+1,0),10,'0')
            INTO   V_CDSEQUENCIAL
            FROM   EVENTOAEV;
        
            INSERT INTO EVENTOAEV 
            (CDEVENTOAEV, NMEVENTOAEV, IDTPEVENAEV, CDOPERINCL, DTINCLEVEN, NMOBJBDEVEN)
            VALUES
            (V_CDSEQUENCIAL,'ETL Comercial Contato','STP','000000000999',SYSDATE,'ETL_COMERCIALTEK');
            
            INSERT INTO PARAMEVENTO
            (CDEVENTOAEV,NMPARAMEVEN,IDDADPARAMEVEN,IDTIPPARAMEVEN,DSPARAMEVEN,IDPARAMNULL,VRTAMPARAM,NRSEQPARAMEVEN) 
            VALUES
            (V_CDSEQUENCIAL,'P_DTINI','D','E','Data Inicial','N',0,'01');
            
            INSERT INTO PARAMEVENTO
            (CDEVENTOAEV,NMPARAMEVEN,IDDADPARAMEVEN,IDTIPPARAMEVEN,DSPARAMEVEN,IDPARAMNULL,VRTAMPARAM,NRSEQPARAMEVEN) 
            VALUES
            (V_CDSEQUENCIAL,'P_DTFINA','D','E','Data Final','N',0,'02');
        
            COMMIT;
        EXCEPTION
           WHEN OTHERS THEN
              RAISE_APPLICATION_ERROR (-20001,'Falha no cadastro de eventos agendveis.');
        END;    
    END IF;
END;
/

/*Cadastrando os fatos e dimenses para o DW*/
BEGIN   
   CRIA_FATOS('ETLCOMERCIALDT', 'QTCONTETLCOM',     'Quantidade de Contatos',            'Comercial');   
       
   CRIA_DIME('ETLCOMERCIALDT',   'CDFILIAL',        'Filial',                            'FILIAL',       'CDFILIAL',       'NMFILIAL',      NULL); 
   CRIA_DIME('ETLCOMERCIALDT',   'CDMOTICONT',      'Motivo do Contato',                 'MOTICONT',     'CDMOTICONT',     'NMMOTICONT',    NULL);    
   CRIA_DIME('ETLCOMERCIALDT',   'CDSISTEMA',       'Sistema',                           'SISTEMA',      'CDSISTEMA',      'NMSISTEMA',     NULL);
   CRIA_DIME('ETLCOMERCIALDT',   'CDOPEREVOL',      'Operador da Evoluo',              'OPERADOR',     'CDOPERADOR',     'NMOPERADOR',    NULL);  
   CRIA_DIME('ETLCOMERCIALDT',   'CDMOTIEVOCON',    'Motivo de Evoluo',                'MOTIEVOLCONT', 'CDMOTIEVOCON',   'NMMOTIEVOCON',  NULL);     
   CRIA_DIME('ETLCOMERCIALDT',   'CDPAIS',          'Pas',                              'PAIS',         'CDPAIS',         'NMPAIS',        NULL);         
   CRIA_DIME('ETLCOMERCIALDT',   'SGESTADO',        'Estado',                            'ESTADMUNICIP', 'SGESTADO',       'NMESTADO',      NULL);         
   CRIA_DIME('ETLCOMERCIALDT',   'CDMUNICIPIO',     'Municpio',                         'ESTADMUNICIP', 'CDMUNICIPIO' ,   'NMMUNICIPIO',   NULL);         
   CRIA_DIME('ETLCOMERCIALDT',   'CDPROCOMERCI',    'Produto Comercial',                 'PROCOMERCI',   'CDPROCOMERCI',   'NMPROCOMERCI',  NULL);
   CRIA_DIME('ETLCOMERCIALDT',   'CDMTENCECONT',    'Motivo de Encerramento',            'MOTIENCECONT', 'CDMTENCECONT',   'DSMTENCECONT',  NULL);
   CRIA_DIME('ETLCOMERCIALDT',   'DSCONTAPRETLCOM', 'Status de Aprovao',                NULL,           NULL,             NULL,           NULL);
   CRIA_DIME('ETLCOMERCIALDT',   'DSSTATETLCOM',    'Status do Contato',                  NULL,           NULL,             NULL,           NULL);
   CRIA_DIME('ETLCOMERCIALDT',   'DSTPCONTETLCOM',  'Tipo de Contato',                    NULL,           NULL,             NULL,           NULL);            
   CRIA_DIME('ETLCOMERCIALDT',   'ANFATETLCOM',     'Ano do Contato',                     NULL,           NULL,             NULL,           NULL);
   CRIA_DIME('ETLCOMERCIALDT',   'NMCONTATOETLCOM', 'Nome do Contato',                    NULL,           NULL,             NULL,           NULL);
   CRIA_DIME('ETLCOMERCIALDT',   'DSPERIFATETLCOM', 'Perodo do Contato',                 NULL,           NULL,             NULL,           NULL);
   
   COMMIT; 
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('ETLCOMERCIALTEK','5.05.002'); END;
/
